# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko 


class CAMSChecks(Checks):
    """
    Checks specific to Cams Global Nrealtime
    """
    def __init__(self, indir, file):
        super().__init__(indir, file)
        self.dataset = "cams_global_nrealtime"

    def check_variable_names(self):
        ds = xr.open_dataset(self.filepath)
        variable_name = self.file.split("_")[0]
        
        try: 
            ds["lon"]
        except KeyError:
            return "Longitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds["lat"]
        except KeyError:
            return "Latitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds["time"]
        except KeyError:
            return "Time variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds[variable_name]
        except KeyError:
            return "{} variable doesn't exist in file - {}".format(variable_name, self.filepath)
        
        
        if ds[variable_name].min() < 0:
            return "File has values less than zero - {}".format(self.filepath)
        
        if ds[variable_name].max() > 100:
            return "File has maximum outliers - {}".format(self.filepath)
        

        

    
    def cams_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)
    
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")


    def cams_global_formatted_files_check(self):
        """
        Checking formatted Cams Global files
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size()
                if file_size != None:
                    error_list.append(file_size)
                    self.send_email_over_ssh(error_list)

                file_variables = self.check_variable_names()
                if file_variables != None:
                    error_list.append(file_variables)
                    self.send_email_over_ssh(error_list)
    
                if error_list != []:
                    print(error_list)
                else:
                    print("No errors reported")




if __name__ == "__main__":
    print(sys.argv)

    #options = getOptions(sys.argv[1:])
    today = datetime.today()
    yesterday = (today - timedelta(days=1)).strftime("%Y%m%d")
    
    


    indir = "/esarchive/recon/ecmwf/cams_global_nrealtime/original_files/0p40_00/3hourly"
    elements = ["lnsp", "odpm", "sconcduT"]
    expid = "0001"
    for element in elements:
        file = "{}_every3hr_{}_{}.nc".format(expid, element, yesterday)
        aer_ori = CAMSChecks(indir, file)
        aer_ori.cams_original_files_check()

    
    indir = "/esarchive/recon/ecmwf/cams_global_nrealtime_cntrl/original_files/0p40_00/3hourly"
    elements = ["lnsp", "odpm", "sconcduT", "depddu", "depwdu"]
    expid = "hylz"
    for element in elements:
        file = "{}_every3hr_{}_{}.nc".format(expid, element, yesterday)
        aer_ori = CAMSChecks(indir, file)
        aer_ori.cams_original_files_check()
    

    # osuite
    # Formatted file check
    path_dir = "/esarchive/recon/ecmwf/cams_global_nrealtime/global/3hourly"
    elements = ["od550aero", "od550du",  "od550ss",  "pm10",  "pm10du",  "pm2p5",  "pm2p5du",  "sconcdu",  "sec550du"]
    for element in elements:
        indir = path_dir + "/{}".format(element)
        file = "{}_{}00.nc".format(element, yesterday)
        print("Checking file {}".format(file))
        aer_fmt = CAMSChecks(indir, file)
        aer_fmt.cams_global_formatted_files_check()
    
    # cntrl
    # Formatted file check
    path = "/esarchive/recon/ecmwf/cams_global_nrealtime_cntrl/global/3hourly"
    elements = ["od550aero", "od550du",  "od550ss",  "pm10",  "pm10du",  "pm2p5",  "pm2p5du",  "sconcdu",  "sec550du", "drydu", "wetdu"]
    for element in elements:
        indir = path + "/{}".format(element)
        file = "{}_{}00.nc".format(element, yesterday)
        print("Checking file {}".format(file))
        aer_fmt = CAMSChecks(indir, file)
        aer_fmt.cams_global_formatted_files_check()

    
    sys.exit()