# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko 


gfas_variables = ['apb',
    'apt',
    'bcfire',
    'cfire',
    'c2h4fire',
    'c2h4ofire',
    'c2h5ohfire',
    'c2h6fire',
    'c2h6sfire',
    'c3h6fire',
    'c3h6ofire',
    'c3h8fire',
    'c4h10fire',
    'c4h8fire',
    'c5h10fire',
    'c5h12fire',
    'c5h8fire',
    'c6h12fire',
    'c6h14fire',
    'c6h6fire',
    'c7h16fire',
    'c7h8fire',
    'c8h10fire',
    'c8h16fire',
    'ch2ofire',
    'ch3ohfire',
    'ch4fire',
    'cofire',
    'co2fire',
    'crfire',
    'frpfire',
    'frpdayfire',
    'frpngtfire',
    'h2fire',
    'hialkanesfire',
    'hialkenesfire',
    'n2ofire',
    'nh3fire',
    'nmhcfire',
    'noxfire',
    'ocfire',
    'offire',
    'pm2p5fire',
    'so2fire',
    'tcfire',
    'terpenesfire',
    'toluenefire',
    'tpmfire']


class GFASChecks(Checks):
    """
    Checks specific to GFAS, this could possibly go in a separate file 
    """
    def __init__(self, indir, file):
        super().__init__(indir, file)
        self.dataset = "GFAS"

    def GFAS_formatted_files_investigation(self):
        """
        Check to see that the file looks okay, i.e.:
         - time = 1 ;
	     - lat = 1800 ;
	     - lon = 3600 ;
	     - nv = 2 ;
        """
        ds = xr.open_dataset(self.filepath)
        print(ds.dims)
        error_list = []
        if ds.dims["time"] != 1:
            error_list.append("File does not have time dimension = 1, has time dimention {}".format(ds.dims['time']))
        if ds.dims["lon"] != 3600:
            error_list.append("File does not have longitude dimension = 3600, has longitude dimention {}".format(ds.dims['lon'])) 
        if ds.dims["lat"] != 1800:
            error_list.append("File does not have latitude dimension = 1800, has latitude dimention {}".format(ds.dims['lat']))
        
        return error_list
 
    def GFAS_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:

                file_size = self.check_file_size(1000)
                if file_size != None:
                    error_list.append(file_size)

    
                if error_list != []:
                    self.send_email_over_ssh(error_list)
                else:
                    print("No errors reported")


    def GFAS_formatted_files_check(self):
        """
        Checking formatted GFAS files at 
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size(10000000) #10MB
                if file_size != None:
                    error_list.append(file_size)
                # Check the file for correct dimensions

                files_inv = self.GFAS_formatted_files_investigation()
                if files_inv != []:
                    error_list.extend(files_inv)
    
                if error_list != []:
                    print(error_list)
                    self.send_email_over_ssh(error_list)
                    return True
                else:
                    print("No errors reported")
                    return False



if __name__ == "__main__":
    print(sys.argv)

    #options = getOptions(sys.argv[1:])
    today = datetime.today()

    # Original files test
    for i in range(14, 16):
        date = (today - timedelta(hours=i)).strftime("%Y%m%d%H")
        path = "/esarchive/recon/ecmwf/gfas/original_files/ftp"
        for variable in gfas_variables:
            indir = path + "/{}/{}".format(date[:8], date[-2:])
            file = "z_cams_c_ecmf_{}00_gfas_an_sfc_001_{}.nc".format(date, variable)
            print("Checking file {}".format(file))
            gfas_ori = GFASChecks(indir, file)
            gfas_ori.GFAS_original_files_check()

    # Formatted file check
    for i in range(14, 100):
        date = (today - timedelta(hours=i)).strftime("%Y%m%d%H")
        path = "/esarchive/recon/ecmwf/gfas/1hourly/multivar"
        indir = path + "/{}".format(date[:8])
        file = "ga_{}_{}.nc".format(date[:8], date[-2:])
        print("Checking file {}".format(file))
        gfas_fmt = GFASChecks(indir, file)
        format_result = gfas_fmt.GFAS_formatted_files_check()
        if format_result:
            old_name = os.path.join(indir, file)
            new_name = os.path.join(indir, "ga_{}_{}_error.nc".format(date[:8], date[-2:]))
            os.rename(old_name, new_name)

            path = "/esarchive/recon/ecmwf/gfas/original_files/ftp"
            for variable in gfas_variables:
                indir = path + "/{}/{}".format(date[:8], date[-2:])
                file = "z_cams_c_ecmf_{}00_gfas_an_sfc_001_{}.nc".format(date, variable)
                os.remove(os.path.join(indir, file))
    
    sys.exit()